@@ -6,44 +6,51 @@
 #   Factory module for constructing templates from Perl code.
 #
 # AUTHOR
-#   Andy Wardley   <abw@kfs.org>
+#   Andy Wardley   <abw@wardley.org>
 #
 # WARNING
 #   Much of this module is hairy, even furry in places.  It needs
 #   a lot of tidying up and may even be moved into a different place 
-#   altogether.  The generator code is often inefficient, particulary in 
+#   altogether.  The generator code is often inefficient, particularly in 
 #   being very anal about pretty-printing the Perl code all neatly, but 
 #   at the moment, that's still high priority for the sake of easier
 #   debugging.
 #
 # COPYRIGHT
-#   Copyright (C) 1996-2000 Andy Wardley.  All Rights Reserved.
-#   Copyright (C) 1998-2000 Canon Research Centre Europe Ltd.
+#   Copyright (C) 1996-2007 Andy Wardley.  All Rights Reserved.
 #
 #   This module is free software; you can redistribute it and/or
 #   modify it under the same terms as Perl itself.
 #
-#----------------------------------------------------------------------------
-#
-# $Id: Directive.pm,v 2.12 2001/06/25 10:55:02 abw Exp $
-#
 #============================================================================
 
 package Template::Directive;
 
-require 5.004;
-
 use strict;
-use vars qw( $VERSION $DEBUG $PRETTY $WHILE_MAX );
+use warnings;
+use base 'Template::Base';
 use Template::Constants;
 use Template::Exception;
 
-$VERSION = sprintf("%d.%02d", q$Revision: 2.12 $ =~ /(\d+)\.(\d+)/);
+our $VERSION   = 2.20;
+our $DEBUG     = 0 unless defined $DEBUG;
+our $WHILE_MAX = 1000 unless defined $WHILE_MAX;
+our $PRETTY    = 0 unless defined $PRETTY;
+our $OUTPUT    = '$output .= ';
 
-$WHILE_MAX = 1000 unless defined $WHILE_MAX;
-$PRETTY    = 0 unless defined $PRETTY;
-my $OUTPUT = '$output .= ';
 
+sub _init {
+    my ($self, $config) = @_;
+    $self->{ NAMESPACE } = $config->{ NAMESPACE };
+    return $self;
+}
+
+sub trace_vars {
+    my $self = shift;
+    return @_
+        ? ($self->{ TRACE_VARS } = shift)
+        :  $self->{ TRACE_VARS };
+}
 
 sub pad {
     my ($text, $pad) = @_;
@@ -63,7 +70,7 @@ sub pad {
 #------------------------------------------------------------------------
 
 sub template {
-    my ($class, $block) = @_;
+    my ($self, $block) = @_;
     $block = pad($block, 2) if $PRETTY;
 
     return "sub { return '' }" unless $block =~ /\S/;
@@ -73,14 +80,14 @@ sub {
     my \$context = shift || die "template sub called without context\\n";
     my \$stash   = \$context->stash;
     my \$output  = '';
-    my \$error;
+    my \$_tt_error;
     
     eval { BLOCK: {
 $block
     } };
     if (\$@) {
-        \$error = \$context->catch(\$@, \\\$output);
-	die \$error unless \$error->type eq 'return';
+        \$_tt_error = \$context->catch(\$@, \\\$output);
+        die \$_tt_error unless \$_tt_error->type eq 'return';
     }
 
     return \$output;
@@ -94,7 +101,7 @@ EOF
 #------------------------------------------------------------------------
 
 sub anon_block {
-    my ($class, $block) = @_;
+    my ($self, $block) = @_;
     $block = pad($block, 2) if $PRETTY;
 
     return <<EOF;
@@ -102,14 +109,14 @@ sub anon_block {
 # BLOCK
 $OUTPUT do {
     my \$output  = '';
-    my \$error;
+    my \$_tt_error;
     
     eval { BLOCK: {
 $block
     } };
     if (\$@) {
-        \$error = \$context->catch(\$@, \\\$output);
-        die \$error unless \$error->type eq 'return';
+        \$_tt_error = \$context->catch(\$@, \\\$output);
+        die \$_tt_error unless \$_tt_error->type eq 'return';
     }
 
     \$output;
@@ -123,7 +130,7 @@ EOF
 #------------------------------------------------------------------------
 
 sub block {
-    my ($class, $block) = @_;
+    my ($self, $block) = @_;
     return join("\n", @{ $block || [] });
 }
 
@@ -133,8 +140,8 @@ sub block {
 #------------------------------------------------------------------------
 
 sub textblock {
-    my ($class, $text) = @_;
-    return "$OUTPUT " . &text($class, $text) . ';';
+    my ($self, $text) = @_;
+    return "$OUTPUT " . &text($self, $text) . ';';
 }
 
 
@@ -143,10 +150,10 @@ sub textblock {
 #------------------------------------------------------------------------
 
 sub text {
-    my ($class, $text) = @_;
+    my ($self, $text) = @_;
     for ($text) {
-	s/(["\$\@\\])/\\$1/g;
-	s/\n/\\n/g;
+        s/(["\$\@\\])/\\$1/g;
+        s/\n/\\n/g;
     }
     return '"' . $text . '"';
 }
@@ -157,7 +164,7 @@ sub text {
 #------------------------------------------------------------------------
 
 sub quoted {
-    my ($class, $items) = @_;
+    my ($self, $items) = @_;
     return '' unless @$items;
     return ("('' . " . $items->[0] . ')') if scalar @$items == 1;
     return '(' . join(' . ', @$items) . ')';
@@ -172,13 +179,48 @@ sub quoted {
 #------------------------------------------------------------------------
 
 sub ident {
-    my ($class, $ident) = @_;
+    my ($self, $ident) = @_;
     return "''" unless @$ident;
+    my $ns;
+
+    # Careful!  Template::Parser always creates a Template::Directive object
+    # (as of v2.22_1) so $self is usually an object.  However, we used to 
+    # allow Template::Directive methods to be called as class methods and 
+    # Template::Namespace::Constants module takes advantage of this fact
+    # by calling Template::Directive->ident() when it needs to generate an
+    # identifier.  This hack guards against Mr Fuckup from coming to town
+    # when that happens.
+    
+    if (ref $self) {
+        # trace variable usage
+        if ($self->{ TRACE_VARS }) {
+            my $root = $self->{ TRACE_VARS };
+            my $n    = 0;
+            my $v;
+            while ($n < @$ident) {
+                $v = $ident->[$n];
+                for ($v) { s/^'//; s/'$// };
+                $root = $root->{ $v } ||= { };
+                $n += 2;
+            }
+        }
+
+        # does the first element of the identifier have a NAMESPACE
+        # handler defined?
+        if (@$ident > 2 && ($ns = $self->{ NAMESPACE })) {
+            my $key = $ident->[0];
+            $key =~ s/^'(.+)'$/$1/s;
+            if ($ns = $ns->{ $key }) {
+                return $ns->ident($ident);
+            }
+        }
+    }
+        
     if (scalar @$ident <= 2 && ! $ident->[1]) {
-	$ident = $ident->[0];
+        $ident = $ident->[0];
     }
     else {
-	$ident = '[' . join(', ', @$ident) . ']';
+        $ident = '[' . join(', ', @$ident) . ']';
     }
     return "\$stash->get($ident)";
 }
@@ -188,13 +230,13 @@ sub ident {
 #------------------------------------------------------------------------
 
 sub identref {
-    my ($class, $ident) = @_;
+    my ($self, $ident) = @_;
     return "''" unless @$ident;
     if (scalar @$ident <= 2 && ! $ident->[1]) {
-	$ident = $ident->[0];
+        $ident = $ident->[0];
     }
     else {
-	$ident = '[' . join(', ', @$ident) . ']';
+        $ident = '[' . join(', ', @$ident) . ']';
     }
     return "\$stash->getref($ident)";
 }
@@ -205,15 +247,15 @@ sub identref {
 #------------------------------------------------------------------------
 
 sub assign {
-    my ($class, $var, $val, $default) = @_;
+    my ($self, $var, $val, $default) = @_;
 
     if (ref $var) {
-	if (scalar @$var == 2 && ! $var->[1]) {
-	    $var = $var->[0];
-	}
-	else {
-	    $var = '[' . join(', ', @$var) . ']';
-	}
+        if (scalar @$var == 2 && ! $var->[1]) {
+            $var = $var->[0];
+        }
+        else {
+            $var = '[' . join(', ', @$var) . ']';
+        }
     }
     $val .= ', 1' if $default;
     return "\$stash->set($var, $val)";
@@ -225,10 +267,10 @@ sub assign {
 #------------------------------------------------------------------------
 
 sub args {
-    my ($class, $args) = @_;
+    my ($self, $args) = @_;
     my $hash = shift @$args;
     push(@$args, '{ ' . join(', ', @$hash) . ' }')
-	if @$hash;
+        if @$hash;
 
     return '0' unless @$args;
     return '[ ' . join(', ', @$args) . ' ]';
@@ -239,12 +281,12 @@ sub args {
 #------------------------------------------------------------------------
 
 sub filenames {
-    my ($class, $names) = @_;
+    my ($self, $names) = @_;
     if (@$names > 1) {
-	$names = '[ ' . join(', ', @$names) . ' ]';
+        $names = '[ ' . join(', ', @$names) . ' ]';
     }
     else {
-	$names = shift @$names;
+        $names = shift @$names;
     }
     return $names;
 }
@@ -255,7 +297,7 @@ sub filenames {
 #------------------------------------------------------------------------
 
 sub get {
-    my ($class, $expr) = @_;  
+    my ($self, $expr) = @_;  
     return "$OUTPUT $expr;";
 }
 
@@ -265,7 +307,7 @@ sub get {
 #------------------------------------------------------------------------
 
 sub call {
-    my ($class, $expr) = @_;  
+    my ($self, $expr) = @_;  
     $expr .= ';';
     return $expr;
 }
@@ -276,10 +318,10 @@ sub call {
 #------------------------------------------------------------------------
 
 sub set {
-    my ($class, $setlist) = @_;
+    my ($self, $setlist) = @_;
     my $output;
     while (my ($var, $val) = splice(@$setlist, 0, 2)) {
-	$output .= &assign($class, $var, $val) . ";\n";
+        $output .= &assign($self, $var, $val) . ";\n";
     }
     chomp $output;
     return $output;
@@ -291,10 +333,10 @@ sub set {
 #------------------------------------------------------------------------
 
 sub default {
-    my ($class, $setlist) = @_;  
+    my ($self, $setlist) = @_;  
     my $output;
     while (my ($var, $val) = splice(@$setlist, 0, 2)) {
-	$output .= &assign($class, $var, $val, 1) . ";\n";
+        $output .= &assign($self, $var, $val, 1) . ";\n";
     }
     chomp $output;
     return $output;
@@ -307,9 +349,9 @@ sub default {
 #------------------------------------------------------------------------
 
 sub insert {
-    my ($class, $nameargs) = @_;
+    my ($self, $nameargs) = @_;
     my ($file, $args) = @$nameargs;
-    $file = $class->filenames($file);
+    $file = $self->filenames($file);
     return "$OUTPUT \$context->insert($file);"; 
 }
 
@@ -320,10 +362,10 @@ sub insert {
 #------------------------------------------------------------------------
 
 sub include {
-    my ($class, $nameargs) = @_;
+    my ($self, $nameargs) = @_;
     my ($file, $args) = @$nameargs;
     my $hash = shift @$args;
-    $file = $class->filenames($file);
+    $file = $self->filenames($file);
     $file .= @$hash ? ', { ' . join(', ', @$hash) . ' }' : '';
     return "$OUTPUT \$context->include($file);"; 
 }
@@ -335,10 +377,10 @@ sub include {
 #------------------------------------------------------------------------
 
 sub process {
-    my ($class, $nameargs) = @_;
+    my ($self, $nameargs) = @_;
     my ($file, $args) = @$nameargs;
     my $hash = shift @$args;
-    $file = $class->filenames($file);
+    $file = $self->filenames($file);
     $file .= @$hash ? ', { ' . join(', ', @$hash) . ' }' : '';
     return "$OUTPUT \$context->process($file);"; 
 }
@@ -353,7 +395,7 @@ sub process {
 #------------------------------------------------------------------------
 
 sub if {
-    my ($class, $expr, $block, $else) = @_;
+    my ($self, $expr, $block, $else) = @_;
     my @else = $else ? @$else : ();
     $else = pop @else;
     $block = pad($block, 1) if $PRETTY;
@@ -361,13 +403,13 @@ sub if {
     my $output = "if ($expr) {\n$block\n}\n";
 
     foreach my $elsif (@else) {
-	($expr, $block) = @$elsif;
-	$block = pad($block, 1) if $PRETTY;
-	$output .= "elsif ($expr) {\n$block\n}\n";
+        ($expr, $block) = @$elsif;
+        $block = pad($block, 1) if $PRETTY;
+        $output .= "elsif ($expr) {\n$block\n}\n";
     }
     if (defined $else) {
-	$else = pad($else, 1) if $PRETTY;
-	$output .= "else {\n$else\n}\n";
+        $else = pad($else, 1) if $PRETTY;
+        $output .= "else {\n$else\n}\n";
     }
 
     return $output;
@@ -381,23 +423,24 @@ sub if {
 #------------------------------------------------------------------------
 
 sub foreach {
-    my ($class, $target, $list, $args, $block) = @_;
+    my ($self, $target, $list, $args, $block, $label) = @_;
     $args  = shift @$args;
     $args  = @$args ? ', { ' . join(', ', @$args) . ' }' : '';
+    $label ||= 'LOOP';
 
     my ($loop_save, $loop_set, $loop_restore, $setiter);
     if ($target) {
-	$loop_save    = 'eval { $oldloop = ' . &ident($class, ["'loop'"]) . ' }';
-	$loop_set     = "\$stash->{'$target'} = \$value";
-	$loop_restore = "\$stash->set('loop', \$oldloop)";
+        $loop_save    = 'eval { $_tt_oldloop = ' . &ident($self, ["'loop'"]) . ' }';
+        $loop_set     = "\$stash->{'$target'} = \$_tt_value";
+        $loop_restore = "\$stash->set('loop', \$_tt_oldloop)";
     }
     else {
-	$loop_save    = '$stash = $context->localise()';
-#	$loop_set     = "\$stash->set('import', \$value) "
-#	                . "if ref \$value eq 'HASH'";
-	$loop_set     = "\$stash->get(['import', [\$value]]) "
-	                . "if ref \$value eq 'HASH'";
-	$loop_restore = '$stash = $context->delocalise()';
+        $loop_save    = '$stash = $context->localise()';
+#       $loop_set     = "\$stash->set('import', \$_tt_value) "
+#                       . "if ref \$value eq 'HASH'";
+        $loop_set     = "\$stash->get(['import', [\$_tt_value]]) "
+                        . "if ref \$_tt_value eq 'HASH'";
+        $loop_restore = '$stash = $context->delocalise()';
     }
     $block = pad($block, 3) if $PRETTY;
 
@@ -405,28 +448,28 @@ sub foreach {
 
 # FOREACH 
 do {
-    my (\$value, \$error, \$oldloop);
-    my \$list = $list;
+    my (\$_tt_value, \$_tt_error, \$_tt_oldloop);
+    my \$_tt_list = $list;
     
-    unless (UNIVERSAL::isa(\$list, 'Template::Iterator')) {
-	\$list = Template::Config->iterator(\$list)
-	    || die \$Template::Config::ERROR, "\\n"; 
+    unless (UNIVERSAL::isa(\$_tt_list, 'Template::Iterator')) {
+        \$_tt_list = Template::Config->iterator(\$_tt_list)
+            || die \$Template::Config::ERROR, "\\n"; 
     }
 
-    (\$value, \$error) = \$list->get_first();
+    (\$_tt_value, \$_tt_error) = \$_tt_list->get_first();
     $loop_save;
-    \$stash->set('loop', \$list);
+    \$stash->set('loop', \$_tt_list);
     eval {
-LOOP:	while (! \$error) {
-	    $loop_set;
+$label:   while (! \$_tt_error) {
+            $loop_set;
 $block;
-	    (\$value, \$error) = \$list->get_next();
-	}
+            (\$_tt_value, \$_tt_error) = \$_tt_list->get_next();
+        }
     };
     $loop_restore;
     die \$@ if \$@;
-    \$error = 0 if \$error && \$error eq Template::Constants::STATUS_DONE;
-    die \$error if \$error;
+    \$_tt_error = 0 if \$_tt_error && \$_tt_error eq Template::Constants::STATUS_DONE;
+    die \$_tt_error if \$_tt_error;
 };
 EOF
 }
@@ -438,9 +481,11 @@ EOF
 #------------------------------------------------------------------------
 
 sub next {
+    my ($self, $label) = @_;
+    $label ||= 'LOOP';
     return <<EOF;
-(\$value, \$error) = \$list->get_next();
-next LOOP;
+(\$_tt_value, \$_tt_error) = \$_tt_list->get_next();
+next $label;
 EOF
 }
 
@@ -451,15 +496,15 @@ EOF
 #------------------------------------------------------------------------
 
 sub wrapper {
-    my ($class, $nameargs, $block) = @_;
+    my ($self, $nameargs, $block) = @_;
     my ($file, $args) = @$nameargs;
     my $hash = shift @$args;
 
     local $" = ', ';
 #    print STDERR "wrapper([@$file], { @$hash })\n";
 
-    return $class->multi_wrapper($file, $hash, $block)
-	if @$file > 1;
+    return $self->multi_wrapper($file, $hash, $block)
+        if @$file > 1;
     $file = shift @$file;
 
     $block = pad($block, 1) if $PRETTY;
@@ -479,7 +524,7 @@ EOF
 
 
 sub multi_wrapper {
-    my ($class, $file, $hash, $block) = @_;
+    my ($self, $file, $hash, $block) = @_;
     $block = pad($block, 1) if $PRETTY;
 
     push(@$hash, "'content'", '$output');
@@ -495,7 +540,7 @@ $OUTPUT do {
     my \$output = '';
 $block
     foreach ($file) {
-	\$output = \$context->include(\$_$hash); 
+        \$output = \$context->include(\$_$hash); 
     }
     \$output;
 };
@@ -510,20 +555,21 @@ EOF
 #------------------------------------------------------------------------
 
 sub while {
-    my ($class, $expr, $block) = @_;
+    my ($self, $expr, $block, $label) = @_;
     $block = pad($block, 2) if $PRETTY;
+    $label ||= 'LOOP';
 
     return <<EOF;
 
 # WHILE
 do {
-    my \$failsafe = $WHILE_MAX;
-LOOP:
-    while (--\$failsafe && ($expr)) {
+    my \$_tt_failsafe = $WHILE_MAX;
+$label:
+    while (--\$_tt_failsafe && ($expr)) {
 $block
     }
     die "WHILE loop terminated (> $WHILE_MAX iterations)\\n"
-        unless \$failsafe;
+        unless \$_tt_failsafe;
 };
 EOF
 }
@@ -537,7 +583,7 @@ EOF
 #------------------------------------------------------------------------
 
 sub switch {
-    my ($class, $expr, $case) = @_;
+    my ($self, $expr, $case) = @_;
     my @case = @$case;
     my ($match, $block, $default);
     my $caseblock = '';
@@ -545,13 +591,13 @@ sub switch {
     $default = pop @case;
 
     foreach $case (@case) {
-	$match = $case->[0];
-	$block = $case->[1];
-	$block = pad($block, 1) if $PRETTY;
-	$caseblock .= <<EOF;
-\$match = $match;
-\$match = [ \$match ] unless ref \$match eq 'ARRAY';
-if (grep(/^\$result\$/, \@\$match)) {
+        $match = $case->[0];
+        $block = $case->[1];
+        $block = pad($block, 1) if $PRETTY;
+        $caseblock .= <<EOF;
+\$_tt_match = $match;
+\$_tt_match = [ \$_tt_match ] unless ref \$_tt_match eq 'ARRAY';
+if (grep(/^\\Q\$_tt_result\\E\$/, \@\$_tt_match)) {
 $block
     last SWITCH;
 }
@@ -559,15 +605,15 @@ EOF
     }
 
     $caseblock .= $default
-	if defined $default;
+        if defined $default;
     $caseblock = pad($caseblock, 2) if $PRETTY;
 
 return <<EOF;
 
 # SWITCH
 do {
-    my \$result = $expr;
-    my \$match;
+    my \$_tt_result = $expr;
+    my \$_tt_match;
     SWITCH: {
 $caseblock
     }
@@ -585,7 +631,7 @@ EOF
 #------------------------------------------------------------------------
 
 sub try {
-    my ($class, $block, $catch) = @_;
+    my ($self, $block, $catch) = @_;
     my @catch = @$catch;
     my ($match, $mblock, $default, $final, $n);
     my $catchblock = '';
@@ -594,30 +640,30 @@ sub try {
     $block = pad($block, 2) if $PRETTY;
     $final = pop @catch;
     $final = "# FINAL\n" . ($final ? "$final\n" : '')
-	   . 'die $error if $error;' . "\n" . '$output;';
+           . 'die $_tt_error if $_tt_error;' . "\n" . '$output;';
     $final = pad($final, 1) if $PRETTY;
 
     $n = 0;
     foreach $catch (@catch) {
-	$match = $catch->[0] || do {
-	    $default ||= $catch->[1];
-	    next;
-	};
-	$mblock = $catch->[1];
-	$mblock = pad($mblock, 1) if $PRETTY;
-	push(@$handlers, "'$match'");
-	$catchblock .= $n++ 
-	    ? "elsif (\$handler eq '$match') {\n$mblock\n}\n" 
-	       : "if (\$handler eq '$match') {\n$mblock\n}\n";
+        $match = $catch->[0] || do {
+            $default ||= $catch->[1];
+            next;
+        };
+        $mblock = $catch->[1];
+        $mblock = pad($mblock, 1) if $PRETTY;
+        push(@$handlers, "'$match'");
+        $catchblock .= $n++ 
+            ? "elsif (\$_tt_handler eq '$match') {\n$mblock\n}\n" 
+               : "if (\$_tt_handler eq '$match') {\n$mblock\n}\n";
     }
-    $catchblock .= "\$error = 0;";
+    $catchblock .= "\$_tt_error = 0;";
     $catchblock = pad($catchblock, 3) if $PRETTY;
     if ($default) {
-	$default = pad($default, 1) if $PRETTY;
-	$default = "else {\n    # DEFAULT\n$default\n    \$error = '';\n}";
+        $default = pad($default, 1) if $PRETTY;
+        $default = "else {\n    # DEFAULT\n$default\n    \$_tt_error = '';\n}";
     }
     else {
-	$default = '# NO DEFAULT';
+        $default = '# NO DEFAULT';
     }
     $default = pad($default, 2) if $PRETTY;
 
@@ -627,16 +673,16 @@ return <<EOF;
 # TRY
 $OUTPUT do {
     my \$output = '';
-    my (\$error, \$handler);
+    my (\$_tt_error, \$_tt_handler);
     eval {
 $block
     };
     if (\$@) {
-        \$error = \$context->catch(\$@, \\\$output);
-	die \$error if \$error->type =~ /^return|stop\$/;
-        \$stash->set('error', \$error);
-        \$stash->set('e', \$error);
-        if (defined (\$handler = \$error->select_handler($handlers))) {
+        \$_tt_error = \$context->catch(\$@, \\\$output);
+        die \$_tt_error if \$_tt_error->type =~ /^return|stop\$/;
+        \$stash->set('error', \$_tt_error);
+        \$stash->set('e', \$_tt_error);
+        if (defined (\$_tt_handler = \$_tt_error->select_handler($handlers))) {
 $catchblock
         }
 $default
@@ -653,30 +699,30 @@ EOF
 #------------------------------------------------------------------------
 
 sub throw {
-    my ($class, $nameargs) = @_;
+    my ($self, $nameargs) = @_;
     my ($type, $args) = @$nameargs;
     my $hash = shift(@$args);
     my $info = shift(@$args);
-    $type = shift @$type;	    # uses same parser production as INCLUDE
-				    # etc., which allow multiple names
-				    # e.g. INCLUDE foo+bar+baz
+    $type = shift @$type;           # uses same parser production as INCLUDE
+                                    # etc., which allow multiple names
+                                    # e.g. INCLUDE foo+bar+baz
 
     if (! $info) {
-	$args = "$type, undef";
+        $args = "$type, undef";
     }
     elsif (@$hash || @$args) {
-	local $" = ', ';
-	my $i = 0;
-	$args = "$type, { args => [ " 
-	      . join(', ', $info, @$args) 
-	      . ' ], '
-	      . join(', ', 
-		     (map { "'" . $i++ . "' => $_" } ($info, @$args)),
-		     @$hash)
-	      . ' }';
+        local $" = ', ';
+        my $i = 0;
+        $args = "$type, { args => [ " 
+              . join(', ', $info, @$args) 
+              . ' ], '
+              . join(', ', 
+                     (map { "'" . $i++ . "' => $_" } ($info, @$args)),
+                     @$hash)
+              . ' }';
     }
     else {
-	$args = "$type, $info";
+        $args = "$type, $info";
     }
     
     return "\$context->throw($args, \\\$output);";
@@ -699,7 +745,7 @@ sub clear {
 # NOTE: this is redundant, being hard-coded (for now) into Parser.yp
 #------------------------------------------------------------------------
 
-sub break {
+sub OLD_break {
     return 'last LOOP;';
 }
 
@@ -726,15 +772,15 @@ sub stop {
 #------------------------------------------------------------------------
 
 sub use {
-    my ($class, $lnameargs) = @_;
+    my ($self, $lnameargs) = @_;
     my ($file, $args, $alias) = @$lnameargs;
-    $file = shift @$file;	# same production rule as INCLUDE
+    $file = shift @$file;       # same production rule as INCLUDE
     $alias ||= $file;
-    $args = &args($class, $args);
+    $args = &args($self, $args);
     $file .= ", $args" if $args;
-#    my $set = &assign($class, $alias, '$plugin'); 
+#    my $set = &assign($self, $alias, '$plugin'); 
     return "# USE\n"
-	 . "\$stash->set($alias,\n"
+         . "\$stash->set($alias,\n"
          . "            \$context->plugin($file));";
 }
 
@@ -744,18 +790,18 @@ sub use {
 #------------------------------------------------------------------------
 
 sub view {
-    my ($class, $nameargs, $block, $defblocks) = @_;
+    my ($self, $nameargs, $block, $defblocks) = @_;
     my ($name, $args) = @$nameargs;
     my $hash = shift @$args;
-    $name = shift @$name;	# same production rule as INCLUDE
+    $name = shift @$name;       # same production rule as INCLUDE
     $block = pad($block, 1) if $PRETTY;
 
     if (%$defblocks) {
-	$defblocks = join(",\n", map { "'$_' => $defblocks->{ $_ }" }
-				keys %$defblocks);
-	$defblocks = pad($defblocks, 1) if $PRETTY;
-	$defblocks = "{\n$defblocks\n}";
-	push(@$hash, "'blocks'", $defblocks);
+        $defblocks = join(",\n", map { "'$_' => $defblocks->{ $_ }" }
+                                keys %$defblocks);
+        $defblocks = pad($defblocks, 1) if $PRETTY;
+        $defblocks = "{\n$defblocks\n}";
+        push(@$hash, "'blocks'", $defblocks);
     }
     $hash = @$hash ? '{ ' . join(', ', @$hash) . ' }' : '';
 
@@ -763,16 +809,16 @@ sub view {
 # VIEW
 do {
     my \$output = '';
-    my \$oldv = \$stash->get('view');
-    my \$view = \$context->view($hash);
-    \$stash->set($name, \$view);
-    \$stash->set('view', \$view);
+    my \$_tt_oldv = \$stash->get('view');
+    my \$_tt_view = \$context->view($hash);
+    \$stash->set($name, \$_tt_view);
+    \$stash->set('view', \$_tt_view);
 
 $block
 
-    \$stash->set('view', \$oldv);
-    \$view->seal();
-    \$output;
+    \$stash->set('view', \$_tt_oldv);
+    \$_tt_view->seal();
+#    \$output;     # not used - commented out to avoid warning
 };
 EOF
 }
@@ -783,7 +829,7 @@ EOF
 #------------------------------------------------------------------------
 
 sub perl {
-    my ($class, $block) = @_;
+    my ($self, $block) = @_;
     $block = pad($block, 1) if $PRETTY;
 
     return <<EOF;
@@ -800,14 +846,14 @@ $block
     local(\$Template::Perl::context) = \$context;
     local(\$Template::Perl::stash)   = \$stash;
 
-    my \$result = '';
-    tie *Template::Perl::PERLOUT, 'Template::TieString', \\\$result;
-    my \$save_stdout = select *Template::Perl::PERLOUT;
+    my \$_tt_result = '';
+    tie *Template::Perl::PERLOUT, 'Template::TieString', \\\$_tt_result;
+    my \$_tt_save_stdout = select *Template::Perl::PERLOUT;
 
     eval \$output;
-    select \$save_stdout;
+    select \$_tt_save_stdout;
     \$context->throw(\$@) if \$@;
-    \$result;
+    \$_tt_result;
 };
 EOF
 }
@@ -818,7 +864,7 @@ EOF
 #------------------------------------------------------------------------
 
 sub no_perl {
-    my $class = shift;
+    my $self = shift;
     return "\$context->throw('perl', 'EVAL_PERL not set');";
 }
 
@@ -831,10 +877,10 @@ sub no_perl {
 #------------------------------------------------------------------------
 
 sub rawperl {
-    my ($class, $block, $line) = @_;
+    my ($self, $block, $line) = @_;
     for ($block) {
-	s/^\n+//;
-	s/\n+$//;
+        s/^\n+//;
+        s/\n+$//;
     }
     $block = pad($block, 1) if $PRETTY;
     $line = $line ? " (starting line $line)" : '';
@@ -853,12 +899,12 @@ EOF
 #------------------------------------------------------------------------
 
 sub filter {
-    my ($class, $lnameargs, $block) = @_;
+    my ($self, $lnameargs, $block) = @_;
     my ($name, $args, $alias) = @$lnameargs;
     $name = shift @$name;
-    $args = &args($class, $args);
+    $args = &args($self, $args);
     $args = $args ? "$args, $alias" : ", undef, $alias"
-	if $alias;
+        if $alias;
     $name .= ", $args" if $args;
     $block = pad($block, 1) if $PRETTY;
  
@@ -867,12 +913,12 @@ sub filter {
 # FILTER
 $OUTPUT do {
     my \$output = '';
-    my \$filter = \$context->filter($name)
+    my \$_tt_filter = \$context->filter($name)
               || \$context->throw(\$context->error);
 
 $block
     
-    &\$filter(\$output);
+    &\$_tt_filter(\$output);
 };
 EOF
 }
@@ -883,15 +929,15 @@ EOF
 #------------------------------------------------------------------------
 
 sub capture {
-    my ($class, $name, $block) = @_;
+    my ($self, $name, $block) = @_;
 
     if (ref $name) {
-	if (scalar @$name == 2 && ! $name->[1]) {
-	    $name = $name->[0];
-	}
-	else {
-	    $name = '[' . join(', ', @$name) . ']';
-	}
+        if (scalar @$name == 2 && ! $name->[1]) {
+            $name = $name->[0];
+        }
+        else {
+            $name = '[' . join(', ', @$name) . ']';
+        }
     }
     $block = pad($block, 1) if $PRETTY;
 
@@ -913,28 +959,28 @@ EOF
 #------------------------------------------------------------------------
 
 sub macro {
-    my ($class, $ident, $block, $args) = @_;
+    my ($self, $ident, $block, $args) = @_;
     $block = pad($block, 2) if $PRETTY;
 
     if ($args) {
-	my $nargs = scalar @$args;
-	$args = join(', ', map { "'$_'" } @$args);
-	$args = $nargs > 1 
-	    ? "\@args{ $args } = splice(\@_, 0, $nargs)"
-	    : "\$args{ $args } = shift";
+        my $nargs = scalar @$args;
+        $args = join(', ', map { "'$_'" } @$args);
+        $args = $nargs > 1 
+            ? "\@_tt_args{ $args } = splice(\@_, 0, $nargs)"
+            : "\$_tt_args{ $args } = shift";
 
-	return <<EOF;
+        return <<EOF;
 
 # MACRO
 \$stash->set('$ident', sub {
     my \$output = '';
-    my (%args, \$params);
+    my (%_tt_args, \$_tt_params);
     $args;
-    \$params = shift;
-    \$params = { } unless ref(\$params) eq 'HASH';
-    \$params = { \%args, %\$params };
+    \$_tt_params = shift;
+    \$_tt_params = { } unless ref(\$_tt_params) eq 'HASH';
+    \$_tt_params = { \%_tt_args, %\$_tt_params };
 
-    my \$stash = \$context->localise(\$params);
+    my \$stash = \$context->localise(\$_tt_params);
     eval {
 $block
     };
@@ -946,14 +992,14 @@ EOF
 
     }
     else {
-	return <<EOF;
+        return <<EOF;
 
 # MACRO
 \$stash->set('$ident', sub {
-    my \$params = \$_[0] if ref(\$_[0]) eq 'HASH';
+    my \$_tt_params = \$_[0] if ref(\$_[0]) eq 'HASH';
     my \$output = '';
 
-    my \$stash = \$context->localise(\$params);
+    my \$stash = \$context->localise(\$_tt_params);
     eval {
 $block
     };
@@ -966,7 +1012,58 @@ EOF
 }
 
 
+sub debug {
+    my ($self, $nameargs) = @_;
+    my ($file, $args) = @$nameargs;
+    my $hash = shift @$args;
+    $args  = join(', ', @$file, @$args);
+    $args .= @$hash ? ', { ' . join(', ', @$hash) . ' }' : '';
+    return "$OUTPUT \$context->debugging($args); ## DEBUG ##"; 
+}
+
+
 1;
 
 __END__
 
+=head1 NAME
+
+Template::Directive - Perl code generator for template directives
+
+=head1 SYNOPSIS
+
+    # no user serviceable parts inside
+
+=head1 DESCRIPTION
+
+The C<Template::Directive> module defines a number of methods that
+generate Perl code for the runtime representation of the various 
+Template Toolkit directives.
+
+It is used internally by the L<Template::Parser> module.
+
+=head1 AUTHOR
+
+Andy Wardley E<lt>abw@wardley.orgE<gt> L<http://wardley.org/>
+
+=head1 COPYRIGHT
+
+Copyright (C) 1996-2007 Andy Wardley.  All Rights Reserved.
+
+This module is free software; you can redistribute it and/or
+modify it under the same terms as Perl itself.
+
+=head1 SEE ALSO
+
+L<Template::Parser>
+
+=cut
+
+# Local Variables:
+# mode: perl
+# perl-indent-level: 4
+# indent-tabs-mode: nil
+# End:
+#
+# vim: expandtab shiftwidth=4:
+
